import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
/* eslint-disable @typescript-eslint/no-shadow */
import React, { forwardRef, useContext, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react';
import classNames from 'classnames';
import CalendarDay from './CalendarDay';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
import { compareDay, formatMonthTitle, getMonthEndDay, getNextDay, getPrevDay, t } from './utils';
import { addUnit, setScrollTop } from '../utils';
import { getRect } from '../hooks/get-rect';
var CalendarMonth = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext = useContext(ConfigProviderContext),
    prefixCls = _useContext.prefixCls,
    createNamespace = _useContext.createNamespace;
  var _createNamespace = createNamespace('calendar', prefixCls),
    _createNamespace2 = _slicedToArray(_createNamespace, 1),
    bem = _createNamespace2[0];
  var _useState = useState(false),
    _useState2 = _slicedToArray(_useState, 2),
    visible = _useState2[0],
    setVisible = _useState2[1];
  var showMonthTitle = props.showMonthTitle,
    date = props.date,
    showMark = props.showMark,
    firstDayOfWeek = props.firstDayOfWeek;
  var daysRef = useRef(null);
  var _useState3 = useState(),
    _useState4 = _slicedToArray(_useState3, 2),
    monthRef = _useState4[0],
    setMonthRef = _useState4[1];
  var height = useRef(0);
  useEffect(function () {
    if (monthRef) {
      height.current = getRect(monthRef).height;
    }
  }, [monthRef]);
  var title = useMemo(function () {
    var formatterFn = props.formatMonthTitle || formatMonthTitle;
    return formatterFn(date);
  }, [date]);
  var rowHeight = useMemo(function () {
    return addUnit(props.rowHeight);
  }, [props.rowHeight]);
  var offset = useMemo(function () {
    var realDay = date.getDay();
    if (firstDayOfWeek) {
      return (realDay + 7 - firstDayOfWeek) % 7;
    }
    return realDay;
  }, [date, firstDayOfWeek]);
  var totalDay = useMemo(function () {
    return getMonthEndDay(date.getFullYear(), date.getMonth() + 1);
  }, [date]);
  var shouldRender = useMemo(function () {
    return visible || !props.lazyRender;
  }, [visible, props.lazyRender]);
  var getTitle = function getTitle() {
    return title;
  };
  var onClick = function onClick(item) {
    var _props$onClick;
    (_props$onClick = props.onClick) === null || _props$onClick === void 0 ? void 0 : _props$onClick.call(props, item);
  };
  var getMultipleDayType = function getMultipleDayType(day) {
    var isSelected = function isSelected(date) {
      return props.currentDate.some(function (item) {
        return compareDay(item, date) === 0;
      });
    };
    if (isSelected(day)) {
      var prevDay = getPrevDay(day);
      var nextDay = getNextDay(day);
      var prevSelected = isSelected(prevDay);
      var nextSelected = isSelected(nextDay);
      if (prevSelected && nextSelected) {
        return 'multiple-middle';
      }
      if (prevSelected) {
        return 'end';
      }
      if (nextSelected) {
        return 'start';
      }
      return 'multiple-selected';
    }
    return '';
  };
  var getRangeDayType = function getRangeDayType(day) {
    var _ref = props.currentDate,
      _ref2 = _slicedToArray(_ref, 2),
      startDay = _ref2[0],
      endDay = _ref2[1];
    if (!startDay) {
      return '';
    }
    var compareToStart = compareDay(day, startDay);
    if (!endDay) {
      return compareToStart === 0 ? 'start' : '';
    }
    var compareToEnd = compareDay(day, endDay);
    if (props.allowSameDay && compareToStart === 0 && compareToEnd === 0) {
      return 'start-end';
    }
    if (compareToStart === 0) {
      return 'start';
    }
    if (compareToEnd === 0) {
      return 'end';
    }
    if (compareToStart > 0 && compareToEnd < 0) {
      return 'middle';
    }
    return '';
  };
  var getDayType = function getDayType(day) {
    var type = props.type,
      minDate = props.minDate,
      maxDate = props.maxDate,
      currentDate = props.currentDate;
    if (compareDay(day, minDate) < 0 || compareDay(day, maxDate) > 0) {
      return 'disabled';
    }
    if (Array.isArray(currentDate)) {
      if (type === 'multiple') {
        return getMultipleDayType(day);
      }
      if (type === 'range') {
        return getRangeDayType(day);
      }
    } else if (type === 'single') {
      return compareDay(day, currentDate) === 0 ? 'selected' : '';
    }
    return '';
  };
  var getBottomInfo = function getBottomInfo(dayType) {
    if (props.type === 'range') {
      if (dayType === 'start' || dayType === 'end') {
        return t(dayType);
      }
      if (dayType === 'start-end') {
        return t('startEnd');
      }
    }
    return '';
  };
  var renderTitle = function renderTitle() {
    return showMonthTitle ? /*#__PURE__*/React.createElement("div", {
      className: classNames(bem('month-title'))
    }, title) : null;
  };
  var renderMark = function renderMark() {
    return showMark && shouldRender ? /*#__PURE__*/React.createElement("div", {
      className: classNames(bem('month-mark'))
    }, props.date.getMonth() + 1) : null;
  };
  var placeholders = useMemo(function () {
    var count = Math.ceil((totalDay + offset) / 7);
    return Array(count).fill({
      type: 'placeholder'
    });
  }, [totalDay, offset]);
  var daysList = useMemo(function () {
    var days = [];
    var year = props.date.getFullYear();
    var month = props.date.getMonth();
    for (var day = 1; day <= totalDay; day++) {
      var _date = new Date(year, month, day);
      var type = getDayType(_date);
      var config = {
        date: _date,
        type: type,
        text: day,
        bottomInfo: getBottomInfo(type)
      };
      if (props.formatter) {
        config = props.formatter(config);
      }
      days.push(config);
    }
    return days;
  }, [props.currentDate, props.date, props.formatter]);
  var disabledDays = useMemo(function () {
    return daysList.filter(function (day) {
      return day.type === 'disabled';
    });
  }, [daysList]);
  var scrollToDate = function scrollToDate(body, targetDate) {
    if (daysRef.current) {
      var daysRect = getRect(daysRef.current);
      var totalRows = placeholders.length;
      var currentRow = Math.ceil((targetDate.getDate() + offset) / 7);
      var rowOffset = (currentRow - 1) * daysRect.height / totalRows;
      setScrollTop(body, daysRect.top + rowOffset + body.scrollTop - getRect(body).top);
    }
  };
  var renderDay = function renderDay(item, index) {
    return /*#__PURE__*/React.createElement(CalendarDay, {
      key: String(index),
      item: item,
      index: index,
      color: props.color,
      offset: offset,
      rowHeight: rowHeight,
      onClick: onClick
    });
  };
  var renderDays = function renderDays() {
    return /*#__PURE__*/React.createElement("div", {
      ref: daysRef,
      role: "grid",
      className: classNames(bem('days'))
    }, renderMark(), (shouldRender ? daysList : placeholders).map(renderDay));
  };
  useImperativeHandle(ref, function () {
    return {
      getTitle: getTitle,
      getHeight: function getHeight() {
        return height.current;
      },
      setVisible: setVisible,
      scrollToDate: scrollToDate,
      disabledDays: disabledDays
    };
  });
  return /*#__PURE__*/React.createElement("div", {
    className: classNames(bem('month')),
    ref: setMonthRef
  }, renderTitle(), renderDays());
});
CalendarMonth.defaultProps = {};
export default CalendarMonth;