import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { useContext, useEffect, useMemo, useImperativeHandle, forwardRef } from 'react';
import classnames from 'classnames';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
import { parseFormat } from './utils';
import { useCountDown } from '../hooks/use-count-down';
var CountDown = /*#__PURE__*/forwardRef(function (props, ref) {
  var time = props.time,
    format = props.format,
    autoStart = props.autoStart,
    millisecond = props.millisecond,
    renderChildren = props.renderChildren;
  var _useCountDown = useCountDown({
      time: +time,
      millisecond: millisecond,
      onChange: function onChange(value) {
        var _props$onChange;
        return (_props$onChange = props.onChange) === null || _props$onChange === void 0 ? void 0 : _props$onChange.call(props, value);
      },
      onFinish: props.onFinish
    }),
    start = _useCountDown.start,
    pause = _useCountDown.pause,
    reset = _useCountDown.reset,
    current = _useCountDown.current;
  var timeText = useMemo(function () {
    return parseFormat(format, current);
  }, [format, current]);
  var resetTime = function resetTime() {
    reset(+time);
    if (autoStart) {
      start();
    }
  };
  useEffect(function () {
    resetTime();
  }, [time]);
  useImperativeHandle(ref, function () {
    return {
      start: start,
      pause: pause,
      reset: resetTime
    };
  });
  var _useContext = useContext(ConfigProviderContext),
    prefixCls = _useContext.prefixCls,
    createNamespace = _useContext.createNamespace;
  var _createNamespace = createNamespace('count-down', prefixCls),
    _createNamespace2 = _slicedToArray(_createNamespace, 1),
    bem = _createNamespace2[0];
  return /*#__PURE__*/React.createElement("div", {
    role: "timer",
    className: classnames(bem())
  }, renderChildren ? renderChildren(current) : timeText);
});
CountDown.defaultProps = {
  time: 0,
  format: 'HH:mm:ss',
  autoStart: true,
  millisecond: false
};
export default CountDown;