import _extends from "@babel/runtime/helpers/esm/extends";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
var _excluded = ["value", "formatter", "columnsOrder", "type", "filter", "minDate", "maxDate"];
import React, { forwardRef, useEffect, useImperativeHandle, useMemo, useRef, useState, useCallback } from 'react';
import Picker from '../picker';
import { getMonthEndDay, getTrueValue, times } from './utils';
import { useMount, useUpdateEffect } from '../hooks';
import { isDate } from '../utils/validate/date';
import { padZero } from '../utils';
import { doubleRaf } from '../utils/raf';
var DatePicker = /*#__PURE__*/forwardRef(function (props, ref) {
  var value = props.value,
    formatter = props.formatter,
    columnsOrder = props.columnsOrder,
    datePickerType = props.type,
    filter = props.filter,
    datePickerMinDate = props.minDate,
    datePickerMaxDate = props.maxDate,
    pickerProps = _objectWithoutProperties(props, _excluded);
  var formatValue = function formatValue(date) {
    if (!isDate(date)) {
      date = props.minDate;
    }
    date = Math.max(date, props.minDate.getTime());
    date = Math.min(date, props.maxDate.getTime());
    return new Date(date);
  };
  var pickerRef = useRef(null);
  var _useState = useState(formatValue(value)),
    _useState2 = _slicedToArray(_useState, 2),
    currentDate = _useState2[0],
    setCurrentDate = _useState2[1];
  var currentDateRef = useRef(currentDate);
  var getBoundary = function getBoundary(type, dateValue) {
    var _ref;
    var boundary = props["".concat(type, "Date")];
    var year = boundary.getFullYear();
    var month = 1;
    var date = 1;
    var hour = 0;
    var minute = 0;
    if (type === 'max') {
      month = 12;
      date = getMonthEndDay(dateValue.getFullYear(), dateValue.getMonth() + 1);
      hour = 23;
      minute = 59;
    }
    if (dateValue.getFullYear() === year) {
      month = boundary.getMonth() + 1;
      if (dateValue.getMonth() + 1 === month) {
        date = boundary.getDate();
        if (dateValue.getDate() === date) {
          hour = boundary.getHours();
          if (dateValue.getHours() === hour) {
            minute = boundary.getMinutes();
          }
        }
      }
    }
    return _ref = {}, _defineProperty(_ref, "".concat(type, "Year"), year), _defineProperty(_ref, "".concat(type, "Month"), month), _defineProperty(_ref, "".concat(type, "Date"), date), _defineProperty(_ref, "".concat(type, "Hour"), hour), _defineProperty(_ref, "".concat(type, "Minute"), minute), _ref;
  };
  var ranges = useMemo(function () {
    var _getBoundary = getBoundary('max', currentDateRef.current),
      maxYear = _getBoundary.maxYear,
      maxDate = _getBoundary.maxDate,
      maxMonth = _getBoundary.maxMonth,
      maxHour = _getBoundary.maxHour,
      maxMinute = _getBoundary.maxMinute;
    var _getBoundary2 = getBoundary('min', currentDateRef.current),
      minYear = _getBoundary2.minYear,
      minDate = _getBoundary2.minDate,
      minMonth = _getBoundary2.minMonth,
      minHour = _getBoundary2.minHour,
      minMinute = _getBoundary2.minMinute;
    var result = [{
      type: 'year',
      range: [minYear, maxYear]
    }, {
      type: 'month',
      range: [minMonth, maxMonth]
    }, {
      type: 'day',
      range: [minDate, maxDate]
    }, {
      type: 'hour',
      range: [minHour, maxHour]
    }, {
      type: 'minute',
      range: [minMinute, maxMinute]
    }];
    switch (props.type) {
      case 'date':
        result = result.slice(0, 3);
        break;
      case 'year-month':
        result = result.slice(0, 2);
        break;
      case 'month-day':
        result = result.slice(1, 3);
        break;
      case 'datehour':
        result = result.slice(0, 4);
        break;
      default:
        break;
    }
    if (columnsOrder) {
      var columnsOrderArr = columnsOrder.concat(result.map(function (column) {
        return column.type;
      }));
      result.sort(function (a, b) {
        return columnsOrderArr.indexOf(a.type) - columnsOrderArr.indexOf(b.type);
      });
    }
    return result;
  }, [columnsOrder, props.type]);
  var originColumns = useMemo(function () {
    return ranges.map(function (_ref2) {
      var type = _ref2.type,
        rangeArr = _ref2.range;
      // 根据范围获取每列的值
      var values = times(rangeArr[1] - rangeArr[0] + 1, function (index) {
        return padZero(rangeArr[0] + index);
      });
      if (filter) {
        values = filter(type, values);
      }
      return {
        type: type,
        values: values
      };
    });
  }, [filter, ranges]);
  var columns = useMemo(function () {
    return originColumns.map(function (column) {
      return {
        values: column.values.map(function (e) {
          return formatter(column.type, e);
        })
      };
    });
  }, [originColumns, formatter]);
  var updateColumnValue = useCallback(function () {
    var values = originColumns.map(function (column) {
      switch (column.type) {
        case 'year':
          return formatter('year', "".concat(currentDate.getFullYear()));
        case 'month':
          return formatter('month', padZero(currentDate.getMonth() + 1));
        case 'day':
          return formatter('day', padZero(currentDate.getDate()));
        case 'hour':
          return formatter('hour', padZero(currentDate.getHours()));
        case 'minute':
          return formatter('minute', padZero(currentDate.getMinutes()));
        default:
          // no default
          return null;
      }
    });
    doubleRaf(function () {
      var _pickerRef$current;
      (_pickerRef$current = pickerRef.current) === null || _pickerRef$current === void 0 ? void 0 : _pickerRef$current.setValues(values);
    });
  }, [currentDate, formatter, originColumns]);
  var updateInnerValue = function updateInnerValue(indexes) {
    var type = props.type;
    var getValue = function getValue(datetimePickerColumnType) {
      var index = 0;
      originColumns.forEach(function (column, columnIndex) {
        if (datetimePickerColumnType === column.type) {
          index = columnIndex;
        }
      });
      var values = originColumns[index].values;
      return getTrueValue(values[indexes[index]]);
    };
    var year = null;
    var month = null;
    var day = null;
    if (type === 'month-day') {
      year = (currentDate || props.minDate).getFullYear();
      month = getValue('month');
      day = getValue('day');
    } else {
      year = getValue('year');
      month = getValue('month');
      day = type === 'year-month' ? 1 : getValue('day');
    }
    var maxDay = getMonthEndDay(year, month);
    day = day > maxDay ? maxDay : day;
    var hour = 0;
    var minute = 0;
    if (type === 'datehour') {
      hour = +getValue('hour');
    }
    if (type === 'datetime') {
      hour = +getValue('hour');
      minute = +getValue('minute');
    }
    return formatValue(new Date(year, month - 1, day, hour, minute));
  };
  var onChange = function onChange(val, index) {
    if (pickerRef.current) {
      var _props$onChange;
      var indexes = pickerRef.current.getIndexes();
      var nextValue = updateInnerValue(indexes);
      setCurrentDate(nextValue);
      currentDateRef.current = nextValue;
      (_props$onChange = props.onChange) === null || _props$onChange === void 0 ? void 0 : _props$onChange.call(props, nextValue);
    }
  };
  var onConfirm = function onConfirm() {
    var _props$onConfirm;
    (_props$onConfirm = props.onConfirm) === null || _props$onConfirm === void 0 ? void 0 : _props$onConfirm.call(props, currentDate);
  };
  useMount(function () {
    setTimeout(function () {
      if (pickerRef.current) {
        var _pickerRef$current2;
        var indexes = (_pickerRef$current2 = pickerRef.current) === null || _pickerRef$current2 === void 0 ? void 0 : _pickerRef$current2.getIndexes();
        var nextValue = updateInnerValue(indexes);
        setCurrentDate(nextValue);
        currentDateRef.current = nextValue;
      }
    }, 0);
  });
  useEffect(function () {
    updateColumnValue();
  }, [updateColumnValue]);
  var minDate = props.minDate,
    maxDate = props.maxDate;
  useUpdateEffect(function () {
    var nextValue = formatValue(value);
    if (nextValue && nextValue.valueOf() !== (currentDate === null || currentDate === void 0 ? void 0 : currentDate.valueOf())) {
      setCurrentDate(nextValue);
      currentDateRef.current = nextValue;
    }
  }, [value, filter, minDate, maxDate]);
  useImperativeHandle(ref, function () {
    return {
      getPicker: function getPicker() {
        return pickerRef.current;
      }
    };
  });
  return /*#__PURE__*/React.createElement(Picker, _extends({}, pickerProps, {
    ref: pickerRef,
    columns: columns,
    onChange: onChange,
    onConfirm: onConfirm,
    onCancel: props.onCancel
  }));
});
var currentYear = new Date().getFullYear();
DatePicker.defaultProps = {
  type: 'datetime',
  minDate: new Date(currentYear - 10, 0, 1),
  maxDate: new Date(currentYear + 10, 11, 31),
  formatter: function formatter(type, value) {
    return value;
  }
};
DatePicker.displayName = 'DatePicker';
export default DatePicker;