import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
import React, { forwardRef, useContext, useImperativeHandle, useRef } from 'react';
import classnames from 'classnames';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
import { getZIndexStyle } from '../utils';
import { renderToContainer } from '../utils/dom/renderToContainer';
import { useSetState } from '../hooks';
import useMergedState from '../hooks/use-merged-state';
import { Icon } from '../icon';
import { Cell } from '../cell';
import Popup from '../popup';
import { DropdownMenuContext } from './DropdownMenu';
var DropdownItem = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext = useContext(ConfigProviderContext),
    prefixCls = _useContext.prefixCls,
    createNamespace = _useContext.createNamespace;
  var _createNamespace = createNamespace('dropdown-item', prefixCls),
    _createNamespace2 = _slicedToArray(_createNamespace, 1),
    bem = _createNamespace2[0];
  var parent = useContext(DropdownMenuContext);
  var _useMergedState = useMergedState({
      defaultValue: props.defaultValue,
      value: props.value
    }),
    _useMergedState2 = _slicedToArray(_useMergedState, 2),
    selectedValue = _useMergedState2[0],
    setSelectedValue = _useMergedState2[1];
  var currentValueRef = useRef(selectedValue);
  var _useSetState = useSetState({
      showPopup: false,
      transition: true,
      showWrapper: false
    }),
    _useSetState2 = _slicedToArray(_useSetState, 2),
    state = _useSetState2[0],
    setState = _useSetState2[1];
  var onOpen = function onOpen() {
    var _props$onOpen;
    (_props$onOpen = props.onOpen) === null || _props$onOpen === void 0 ? void 0 : _props$onOpen.call(props);
  };
  var onOpened = function onOpened() {
    var _props$onOpened;
    (_props$onOpened = props.onOpened) === null || _props$onOpened === void 0 ? void 0 : _props$onOpened.call(props);
  };
  var onClose = function onClose() {
    var _props$onClose;
    setState({
      showPopup: false
    });
    parent.close();
    (_props$onClose = props.onClose) === null || _props$onClose === void 0 ? void 0 : _props$onClose.call(props);
  };
  var onClosed = function onClosed() {
    var _props$onClosed;
    setState({
      showWrapper: false
    });
    (_props$onClosed = props.onClosed) === null || _props$onClosed === void 0 ? void 0 : _props$onClosed.call(props);
  };
  var onClickWrapper = function onClickWrapper(event) {
    // 防止在使用teleport时被识别为在外面点击并关闭
    if (props.teleport) {
      event.stopPropagation();
    }
  };
  var toggle = function toggle() {
    var show = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : !state.showPopup;
    var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
    if (show === state.showPopup) {
      return;
    }
    var newState = {};
    newState.showPopup = show;
    newState.transition = !options.immediate;
    if (show) {
      state.showWrapper = true;
    }
    setState(newState);
  };
  var renderTitle = function renderTitle() {
    if (props.title) {
      return props.title;
    }
    var match = props.options.find(function (option) {
      var _currentValueRef$curr;
      return option.value === ((_currentValueRef$curr = currentValueRef.current) === null || _currentValueRef$curr === void 0 ? void 0 : _currentValueRef$curr.value);
    });
    return match ? match.text : props.placeholder;
  };
  useImperativeHandle(ref, function () {
    return {
      toggle: toggle,
      renderTitle: renderTitle,
      state: state,
      name: props.name,
      titleClass: props.titleClass,
      disabled: props.disabled
    };
  });
  var renderOption = function renderOption(option) {
    var _currentValueRef$curr2;
    var activeColor = parent.props.activeColor;
    var active = option.value === ((_currentValueRef$curr2 = currentValueRef.current) === null || _currentValueRef$curr2 === void 0 ? void 0 : _currentValueRef$curr2.value);
    var onClick = function onClick() {
      var _currentValueRef$curr3;
      if (option.value !== ((_currentValueRef$curr3 = currentValueRef.current) === null || _currentValueRef$curr3 === void 0 ? void 0 : _currentValueRef$curr3.value)) {
        var _props$onChange, _parent$onChange;
        var newValue = _objectSpread(_objectSpread({}, selectedValue), {}, {
          text: option.text,
          value: option.value
        });
        setSelectedValue(newValue);
        currentValueRef.current = newValue;
        (_props$onChange = props.onChange) === null || _props$onChange === void 0 ? void 0 : _props$onChange.call(props, newValue);
        (_parent$onChange = parent.onChange) === null || _parent$onChange === void 0 ? void 0 : _parent$onChange.call(parent, newValue);
        onClose();
      }
    };
    var renderIcon = function renderIcon() {
      if (active) {
        return /*#__PURE__*/React.createElement(Icon, {
          className: classnames(bem('icon')),
          color: activeColor,
          name: "success"
        });
      }
      return null;
    };
    return /*#__PURE__*/React.createElement(Cell, {
      key: option.value,
      icon: option.icon,
      title: option.text,
      className: classnames(bem('option', {
        active: active
      })),
      style: {
        color: active ? activeColor : ''
      },
      clickable: true,
      onClick: onClick
    }, renderIcon());
  };
  var renderContent = function renderContent() {
    var _props$options;
    var offset = parent.offset;
    var _parent$props = parent.props,
      zIndex = _parent$props.zIndex,
      overlay = _parent$props.overlay,
      duration = _parent$props.duration,
      direction = _parent$props.direction,
      closeOnClickOverlay = _parent$props.closeOnClickOverlay;
    var style = getZIndexStyle(zIndex);
    if (direction === 'down') {
      style.top = "".concat(offset, "px");
    } else {
      style.bottom = "".concat(offset, "px");
    }
    return /*#__PURE__*/React.createElement("div", {
      style: _objectSpread(_objectSpread({}, style), {}, {
        display: state.showWrapper ? 'block' : 'none'
      }),
      className: classnames(bem([direction])),
      onClick: onClickWrapper
    }, /*#__PURE__*/React.createElement(Popup, {
      visible: state.showPopup,
      className: classnames(bem('content')),
      overlay: overlay,
      position: direction === 'down' ? 'top' : 'bottom',
      duration: state.transition ? +duration : 0,
      overlayStyle: {
        position: 'absolute'
      },
      closeOnClickOverlay: closeOnClickOverlay,
      teleport: null,
      onOpen: onOpen,
      onClose: onClose,
      onOpened: onOpened,
      onClosed: onClosed
    }, (_props$options = props.options) === null || _props$options === void 0 ? void 0 : _props$options.map(renderOption), props.children));
  };
  return props.teleport ? renderToContainer(props.teleport, renderContent()) : renderContent();
});
DropdownItem.defaultProps = {
  options: [],
  disabled: false,
  placeholder: '请选择'
};
DropdownItem.displayName = 'DropdownItem';
export default DropdownItem;