import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import { useMemo, useEffect, useState, useRef } from 'react';
import { inBrowser } from '../utils';
import { cancelRaf, raf } from '../utils/raf';
var SECOND = 1000;
var MINUTE = 60 * SECOND;
var HOUR = 60 * MINUTE;
var DAY = 24 * HOUR;
function parseTime(time) {
  var days = Math.floor(time / DAY);
  var hours = Math.floor(time % DAY / HOUR);
  var minutes = Math.floor(time % HOUR / MINUTE);
  var seconds = Math.floor(time % MINUTE / SECOND);
  var milliseconds = Math.floor(time % SECOND);
  return {
    total: time,
    days: days,
    hours: hours,
    minutes: minutes,
    seconds: seconds,
    milliseconds: milliseconds
  };
}
function isSameSecond(time1, time2) {
  return Math.floor(time1 / 1000) === Math.floor(time2 / 1000);
}
export function useCountDown(options) {
  var _useState = useState(options.time),
    _useState2 = _slicedToArray(_useState, 2),
    remain = _useState2[0],
    setRemain = _useState2[1];
  var remainRef = useRef(remain);
  var rafId = useRef();
  var endTime = useRef();
  var counting = useRef();
  var current = useMemo(function () {
    return parseTime(remain);
  }, [remain]);
  var pause = function pause() {
    counting.current = false;
    cancelRaf(rafId.current);
  };
  var getCurrentRemain = function getCurrentRemain() {
    return Math.max(endTime.current - Date.now(), 0);
  };
  var handleRemain = function handleRemain(value) {
    var _options$onChange;
    setRemain(value);
    remainRef.current = value;
    (_options$onChange = options.onChange) === null || _options$onChange === void 0 ? void 0 : _options$onChange.call(options, parseTime(value));
    if (value === 0) {
      var _options$onFinish;
      pause();
      (_options$onFinish = options.onFinish) === null || _options$onFinish === void 0 ? void 0 : _options$onFinish.call(options);
    }
  };
  var microTick = function microTick() {
    rafId.current = raf(function () {
      // in case of call reset immediately after finish
      if (counting.current) {
        handleRemain(getCurrentRemain());
        if (remainRef.current > 0) {
          microTick();
        }
      }
    });
  };
  var macroTick = function macroTick() {
    rafId.current = raf(function () {
      // in case of call reset immediately after finish
      if (counting.current) {
        var remainRemain = getCurrentRemain();
        if (!isSameSecond(remainRemain, remainRef.current) || remainRemain === 0) {
          handleRemain(remainRemain);
        }
        if (remainRef.current > 0) {
          macroTick();
        }
      }
    });
  };
  var tick = function tick() {
    // should not start counting in server
    if (!inBrowser) {
      return;
    }
    if (options.millisecond) {
      microTick();
    } else {
      macroTick();
    }
  };
  var start = function start() {
    if (!counting.current) {
      endTime.current = Date.now() + remainRef.current;
      counting.current = true;
      tick();
    }
  };
  var reset = function reset() {
    var totalTime = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : options.time;
    pause();
    setRemain(totalTime);
    remainRef.current = totalTime;
  };
  useEffect(function () {
    return function () {
      pause();
    };
  }, []);
  return {
    start: start,
    pause: pause,
    reset: reset,
    current: current
  };
}