import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { useContext, useRef } from 'react';
import classnames from 'classnames';
import Icon from '../icon';
import { getZIndexStyle } from '../utils';
import { BORDER_BOTTOM, HAPTICS_FEEDBACK } from '../utils/constant';
import useHeight from '../hooks/use-height';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
var NavBar = function NavBar(props) {
  var _useContext = useContext(ConfigProviderContext),
    prefixCls = _useContext.prefixCls,
    createNamespace = _useContext.createNamespace;
  var _createNamespace = createNamespace('nav-bar', prefixCls),
    _createNamespace2 = _slicedToArray(_createNamespace, 1),
    bem = _createNamespace2[0];
  var navBarRef = useRef();
  var height = useHeight(navBarRef);
  var onClickLeft = function onClickLeft(event) {
    var _props$onClickLeft;
    return (_props$onClickLeft = props.onClickLeft) === null || _props$onClickLeft === void 0 ? void 0 : _props$onClickLeft.call(props, event);
  };
  var onClickRight = function onClickRight(event) {
    var _props$onClickRight;
    return (_props$onClickRight = props.onClickRight) === null || _props$onClickRight === void 0 ? void 0 : _props$onClickRight.call(props, event);
  };
  var renderLeft = function renderLeft() {
    return [props.leftArrow && /*#__PURE__*/React.createElement(Icon, {
      key: "icon",
      className: classnames(bem('arrow')),
      name: "arrow-left"
    }), props.leftArea && /*#__PURE__*/React.createElement("span", {
      key: "left-area",
      className: classnames(bem('text'))
    }, props.leftArea)];
  };
  var renderRight = function renderRight() {
    return /*#__PURE__*/React.createElement("span", {
      className: classnames(bem('text'))
    }, props.rightArea);
  };
  var renderNavBar = function renderNavBar() {
    var title = props.title,
      fixed = props.fixed,
      border = props.border,
      zIndex = props.zIndex;
    var style = getZIndexStyle(zIndex);
    var hasLeft = props.leftArrow || props.leftArea;
    var hasRight = props.rightArea;
    return /*#__PURE__*/React.createElement("div", {
      ref: navBarRef,
      style: style,
      className: classnames(bem({
        fixed: fixed,
        'safe-area-inset-top': props.safeAreaInsetTop
      }), _defineProperty({}, BORDER_BOTTOM, border))
    }, /*#__PURE__*/React.createElement("div", {
      className: classnames(bem('content'))
    }, hasLeft && /*#__PURE__*/React.createElement("div", {
      className: classnames(bem('left'), HAPTICS_FEEDBACK),
      onClick: onClickLeft
    }, renderLeft()), /*#__PURE__*/React.createElement("div", {
      className: classnames(bem('title'), 'van-ellipsis')
    }, title), hasRight && /*#__PURE__*/React.createElement("div", {
      className: classnames(bem('right'), HAPTICS_FEEDBACK),
      onClick: onClickRight
    }, renderRight())));
  };
  var renderPlaceholder = function renderPlaceholder() {
    return /*#__PURE__*/React.createElement("div", {
      className: classnames(bem('placeholder')),
      style: {
        height: height ? "".concat(height, "px") : undefined
      }
    }, renderNavBar());
  };
  if (props.fixed && props.placeholder) {
    return renderPlaceholder();
  }
  return renderNavBar();
};
NavBar.defaultProps = {
  title: '',
  leftArea: '',
  rightArea: '',
  leftArrow: false,
  border: true,
  fixed: false,
  placeholder: false,
  zIndex: 1,
  safeAreaInsetTop: false
};
export default NavBar;