import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { useContext, useMemo, useRef } from 'react';
import classNames from 'classnames';
import { Popup } from '../popup';
import { genCustomKeys, genDefaultKeys } from './utils';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
import NumberKeyboardKey from './NumberKeyboardKey';
import useClickAway from '../hooks/use-click-away';
var NumberKeyboard = function NumberKeyboard(props) {
  var _useContext = useContext(ConfigProviderContext),
    prefixCls = _useContext.prefixCls,
    createNamespace = _useContext.createNamespace;
  var _createNamespace = createNamespace('number-keyboard', prefixCls),
    _createNamespace2 = _slicedToArray(_createNamespace, 1),
    bem = _createNamespace2[0];
  var visible = props.visible,
    safeAreaInsetBottom = props.safeAreaInsetBottom,
    theme = props.theme,
    blurOnClose = props.blurOnClose,
    showDeleteKey = props.showDeleteKey,
    hideOnClickOutside = props.hideOnClickOutside,
    zIndex = props.zIndex,
    teleport = props.teleport,
    title = props.title,
    closeButtonText = props.closeButtonText,
    titleLeft = props.titleLeft;
  var root = useRef(null);
  var showClose = closeButtonText && theme === 'default';
  var showTitle = !!(title || showClose || titleLeft);
  var keys = useMemo(function () {
    return theme === 'custom' ? genCustomKeys(props) : genDefaultKeys(props);
  }, [theme]);
  var onBlur = function onBlur() {
    if (visible) {
      var _props$onBlur;
      (_props$onBlur = props.onBlur) === null || _props$onBlur === void 0 ? void 0 : _props$onBlur.call(props);
    }
  };
  var onClose = function onClose() {
    var _props$onClose;
    (_props$onClose = props.onClose) === null || _props$onClose === void 0 ? void 0 : _props$onClose.call(props);
    if (blurOnClose) {
      onBlur();
    }
  };
  var onPress = function onPress(text, type) {
    if (text === '') {
      if (type === 'extra') {
        onBlur();
      }
      return;
    }
    if (type === 'delete') {
      var _props$onDelete;
      (_props$onDelete = props.onDelete) === null || _props$onDelete === void 0 ? void 0 : _props$onDelete.call(props);
    } else if (type === 'close') {
      onClose();
    } else {
      var _props$onInput;
      (_props$onInput = props.onInput) === null || _props$onInput === void 0 ? void 0 : _props$onInput.call(props, text);
    }
  };
  var renderTitle = function renderTitle() {
    if (!showTitle) {
      return null;
    }
    return /*#__PURE__*/React.createElement("div", {
      className: classNames(bem('header'))
    }, titleLeft && /*#__PURE__*/React.createElement("span", {
      className: classNames(bem('title-left'))
    }, titleLeft), title && /*#__PURE__*/React.createElement("h2", {
      className: classNames(bem('title'))
    }, title), showClose && /*#__PURE__*/React.createElement("button", {
      type: "button",
      className: classNames(bem('close')),
      onClick: onClose
    }, closeButtonText));
  };
  var renderKeys = function renderKeys() {
    return keys.map(function (key) {
      return /*#__PURE__*/React.createElement(NumberKeyboardKey, {
        key: "".concat(key.text).concat(key.type),
        text: key.text,
        type: key.type,
        wider: key.wider,
        color: key.color,
        onPress: onPress
      });
    });
  };
  var renderSidebar = function renderSidebar() {
    if (theme === 'custom') {
      return /*#__PURE__*/React.createElement("div", {
        className: classNames(bem('sidebar'))
      }, showDeleteKey && /*#__PURE__*/React.createElement(NumberKeyboardKey, {
        large: true,
        text: props.deleteButtonText,
        type: "delete",
        onPress: onPress
      }), /*#__PURE__*/React.createElement(NumberKeyboardKey, {
        large: true,
        text: props.closeButtonText,
        type: "close",
        color: "blue",
        loading: props.closeButtonLoading,
        onPress: onPress
      }));
    }
    return null;
  };
  useClickAway(root, onBlur, 'touchstart', hideOnClickOutside);
  return /*#__PURE__*/React.createElement(Popup, {
    visible: visible,
    preventDefaultMouseDown: true,
    overlay: false,
    round: showTitle,
    zIndex: zIndex,
    position: "bottom",
    duration: 300,
    teleport: teleport,
    safeAreaInsetBottom: safeAreaInsetBottom,
    onClose: onClose
  }, /*#__PURE__*/React.createElement("div", {
    ref: root,
    className: classNames(bem())
  }, renderTitle(), /*#__PURE__*/React.createElement("div", {
    className: classNames(bem('body'))
  }, /*#__PURE__*/React.createElement("div", {
    className: classNames(bem('keys'))
  }, renderKeys()), renderSidebar())));
};
NumberKeyboard.defaultProps = {
  theme: 'default',
  visible: false,
  extraKey: '',
  showDeleteKey: true,
  zIndex: 100,
  closeButtonLoading: false,
  hideOnClickOutside: true,
  randomKeyOrder: false,
  safeAreaInsetBottom: true,
  blurOnClose: true
};
export default NumberKeyboard;