import _extends from "@babel/runtime/helpers/esm/extends";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import React, { forwardRef, useCallback, useContext, useEffect, useImperativeHandle, useMemo, useRef } from 'react';
import { useSpring, animated } from '@react-spring/web';
import { useDrag } from '@use-gesture/react';
import classNames from 'classnames';
import { useSetState, useUpdateEffect } from '../hooks';
import { isObject, range } from '../utils';
import { deepClone } from '../utils/deep-clone';
import { rubberbandIfOutOfBounds } from '../utils/rubberband';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
function isOptionDisabled(option) {
  return isObject(option) && option.disabled;
}
var DEFAULT_DURATION = 200;
var MOMENTUM_LIMIT_DISTANCE = 15;
var PickerColumn = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext = useContext(ConfigProviderContext),
    prefixCls = _useContext.prefixCls,
    createNamespace = _useContext.createNamespace;
  var _createNamespace = createNamespace('picker-column', prefixCls),
    _createNamespace2 = _slicedToArray(_createNamespace, 1),
    bem = _createNamespace2[0];
  var itemHeight = props.itemHeight,
    visibleItemCount = props.visibleItemCount,
    defaultIndex = props.defaultIndex,
    initialOptions = props.initialOptions;
  var root = useRef(null);
  var wrapper = useRef(null);
  var indexRef = useRef(defaultIndex);
  var optionsRef = useRef(deepClone(initialOptions));
  var _useSetState = useSetState({
      offset: 0,
      duration: 0,
      options: deepClone(initialOptions)
    }),
    _useSetState2 = _slicedToArray(_useSetState, 2),
    state = _useSetState2[0],
    updateState = _useSetState2[1];
  var count = useMemo(function () {
    return optionsRef.current.length;
  }, [optionsRef.current.length]);
  var baseOffset = useMemo(function () {
    // 默认转入第一个选项的位置
    return itemHeight * (+visibleItemCount - 1) / 2;
  }, [itemHeight, visibleItemCount]);
  var draggingRef = useRef(false);
  var _useSpring = useSpring(function () {
      return {
        from: {
          y: defaultIndex * -itemHeight + baseOffset
        },
        config: {
          tension: 400,
          mass: 0.8
        }
      };
    }),
    _useSpring2 = _slicedToArray(_useSpring, 2),
    y = _useSpring2[0].y,
    api = _useSpring2[1];
  var adjustIndex = function adjustIndex(index) {
    index = range(index, 0, count);
    for (var i = index; i < count; i += 1) {
      if (!isOptionDisabled(optionsRef.current[i])) return i;
    }
    // for (let i = index - 1; i >= 0; i -= 1) {
    //   if (!isOptionDisabled(optionsRef.current[i])) return i;
    // }

    return null;
  };
  var setIndex = function setIndex(index, emitChange) {
    index = adjustIndex(index) || 0;
    var offset = index * -itemHeight;
    api.start({
      y: offset + baseOffset
    });
    if (index !== indexRef.current) {
      indexRef.current = index;
      // updateState({ index });
      if (emitChange && props.onChange) {
        setTimeout(function () {
          props.onChange(index);
        }, 0);
      }
    }
    updateState({
      offset: offset
    });
  };
  var setOptions = function setOptions(options) {
    if (JSON.stringify(options) !== JSON.stringify(optionsRef.current)) {
      optionsRef.current = options;
      setIndex(props.defaultIndex);
    }
  };
  var onClickItem = function onClickItem(index) {
    if (draggingRef.current || props.readonly) {
      return;
    }
    updateState({
      duration: DEFAULT_DURATION
    });
    setIndex(index, true);
  };
  var getOptionText = function getOptionText(option) {
    if (isObject(option) && props.textKey in option) {
      return option[props.textKey];
    }
    return option;
  };
  var getIndexByOffset = function getIndexByOffset(offset) {
    return range(Math.round(-offset / props.itemHeight), 0, count - 1);
  };
  var bind = useDrag(function (dragState) {
    if (props.readonly) {
      return;
    }
    draggingRef.current = true;
    if (dragState.tap) {
      updateState({
        duration: 0,
        offset: dragState.offset[1]
      });
    } else if (dragState.last) {
      var allowMomentum = Math.abs(dragState.velocity[1] * dragState.direction[1] * 50) > MOMENTUM_LIMIT_DISTANCE;
      var position = dragState.offset[1] + dragState.velocity[1] * dragState.direction[1] * 50 - baseOffset;
      var targetIndex = getIndexByOffset(position);
      updateState({
        duration: allowMomentum ? +props.swipeDuration : DEFAULT_DURATION,
        offset: position + baseOffset
      });
      setIndex(targetIndex, true);
    } else {
      var _position = dragState.offset[1];
      var max = baseOffset;
      var min = -(count * itemHeight) + baseOffset;
      // const offset = range(position, -(count * itemHeight), itemHeight);
      var offset = rubberbandIfOutOfBounds(_position, min, max, itemHeight * 50, 0.2);
      api.start({
        y: offset
      });
    }
  }, {
    axis: 'y',
    from: function from() {
      return [0, y.get()];
    },
    filterTaps: true,
    pointer: {
      touch: true
    }
  });
  var renderOptions = useMemo(function () {
    var optionStyle = {
      height: "".concat(props.itemHeight, "px")
    };
    return optionsRef.current.map(function (option, index) {
      var text = getOptionText(option);
      var disabled = isOptionDisabled(option);
      var data = {
        role: 'button',
        key: index,
        style: optionStyle,
        tabIndex: disabled ? -1 : 0,
        className: classNames(bem('item', {
          disabled: disabled,
          selected: index === indexRef.current
        })),
        onClick: function onClick() {
          draggingRef.current = false;
          onClickItem(index);
        }
      };
      var childData = {
        className: 'rc-ellipsis',
        children: text
      };
      return /*#__PURE__*/React.createElement("li", data, props.optionRender ? props.optionRender(option) : /*#__PURE__*/React.createElement("div", childData));
    });
  }, [optionsRef.current, indexRef.current, onClickItem, props.optionRender]);
  var setValue = function setValue(value) {
    var options = optionsRef.current;
    for (var i = 0; i < options.length; i += 1) {
      if (getOptionText(options[i]) === value) {
        return setIndex(i);
      }
    }
    return null;
  };
  var getValue = useCallback(function () {
    return optionsRef.current[indexRef.current];
  }, [indexRef.current, optionsRef.current]);
  useEffect(function () {
    setIndex(defaultIndex);
  }, [defaultIndex]);
  useUpdateEffect(function () {
    setOptions(deepClone(initialOptions));
  }, [initialOptions]);
  useImperativeHandle(ref, function () {
    return {
      state: state,
      getIndex: function getIndex() {
        return indexRef.current;
      },
      getOptions: function getOptions() {
        return optionsRef.current;
      },
      setIndex: setIndex,
      getValue: getValue,
      setValue: setValue,
      setOptions: setOptions
    };
  });
  return /*#__PURE__*/React.createElement("div", _extends({
    ref: root,
    className: classNames(bem(), props.className)
  }, bind()), /*#__PURE__*/React.createElement(animated.ul, {
    ref: wrapper,
    className: classNames(bem('wrapper')),
    style: {
      y: y,
      transitionDuration: "".concat(state.duration, "ms"),
      transitionProperty: state.duration ? 'all' : 'none'
    },
    "aria-hidden": true
  }, renderOptions));
});
PickerColumn.defaultProps = {
  initialOptions: [],
  defaultIndex: 0
};
PickerColumn.displayName = 'PickerColumn';
export default PickerColumn;