import _extends from "@babel/runtime/helpers/esm/extends";
import _defineProperty from "@babel/runtime/helpers/esm/defineProperty";
import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); enumerableOnly && (symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; })), keys.push.apply(keys, symbols); } return keys; }
function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = null != arguments[i] ? arguments[i] : {}; i % 2 ? ownKeys(Object(source), !0).forEach(function (key) { _defineProperty(target, key, source[key]); }) : Object.getOwnPropertyDescriptors ? Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)) : ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } return target; }
/* eslint-disable no-return-assign */
/* eslint-disable no-plusplus */
import React, { forwardRef, useContext, useEffect, useImperativeHandle, useMemo, useRef, useState } from 'react';
import classnames from 'classnames';
import { useSpring, animated } from '@react-spring/web';
import { useDrag } from '@use-gesture/react';
import ConfigProviderContext from '../config-provider/ConfigProviderContext';
import { PageIndicator } from './page-indicator';
import SwiperItem from './SwiperItem';
import useRefState from '../hooks/use-ref-state';
import { bound } from '../utils/bound';
import { devWarning } from '../utils/dev-log';
import { noop } from '../utils';
import { getRect } from '../hooks/get-rect';
import useMountedRef from '../hooks/use-mounted-ref';
function modulus(value, division) {
  var remainder = value % division;
  return remainder < 0 ? remainder + division : remainder;
}
var Swiper = /*#__PURE__*/forwardRef(function (props, ref) {
  var _useContext = useContext(ConfigProviderContext),
    prefixCls = _useContext.prefixCls,
    createNamespace = _useContext.createNamespace;
  var _createNamespace = createNamespace('swiper', prefixCls),
    _createNamespace2 = _slicedToArray(_createNamespace, 1),
    bem = _createNamespace2[0];
  var outerLoop = props.loop,
    autoplay = props.autoplay,
    direction = props.direction,
    autoplayInterval = props.autoplayInterval;
  var _lock = useRef(false);
  var trackRef = useRef(null);
  var _useState = useState(null),
    _useState2 = _slicedToArray(_useState, 2),
    root = _useState2[0],
    setRoot = _useState2[1];
  var _useState3 = useState(props.defaultIndex),
    _useState4 = _slicedToArray(_useState3, 2),
    current = _useState4[0],
    setCurrent = _useState4[1];
  var _useRefState = useRefState(false),
    _useRefState2 = _slicedToArray(_useRefState, 3),
    dragging = _useRefState2[0],
    setDragging = _useRefState2[1],
    draggingRef = _useRefState2[2];
  var isVertical = direction === 'vertical';
  var axis = useMemo(function () {
    return isVertical ? 'y' : 'x';
  }, [isVertical]);
  var slideRatio = props.slideSize / 100;
  var offsetRatio = props.trackOffset / 100;
  var computedStyle = useMemo(function () {
    var _objectSpread2;
    return _objectSpread((_objectSpread2 = {}, _defineProperty(_objectSpread2, "--".concat(prefixCls, "-swipe-slide-size"), "".concat(props.slideSize, "%")), _defineProperty(_objectSpread2, "--".concat(prefixCls, "-swipe-track-offset"), "".concat(props.trackOffset, "%")), _objectSpread2), props.style);
  }, [props.style, props.slideSize, props.trackOffset]);
  var axisDistance = useMemo(function () {
    if (!isVertical) return 100;
    var rect = getRect(root);
    return rect.height;
  }, [isVertical, root]);
  var _useMemo = useMemo(function () {
      var innerCount = 0;
      var innerValidChildren = React.Children.map(props.children, function (child) {
        if (! /*#__PURE__*/React.isValidElement(child)) return null;
        if (child.type !== SwiperItem) {
          devWarning('Swiper', 'The children of `Swiper` must be `Swiper.Item` components.');
          return null;
        }
        innerCount++;
        return child;
      });
      return {
        validChildren: innerValidChildren,
        count: innerCount
      };
    }, [props.children]),
    validChildren = _useMemo.validChildren,
    count = _useMemo.count;
  var loop = useMemo(function () {
    if (count <= 1) return false;
    return outerLoop;
  }, [count, outerLoop]);
  var mountedRef = useMountedRef();
  var getSlidePixels = function getSlidePixels() {
    var track = trackRef.current;
    if (!track) return 0;
    var trackPixels = isVertical ? track.offsetHeight : track.offsetWidth;
    return trackPixels * props.slideSize / 100;
  };
  var boundIndex = function boundIndex(cur) {
    var min = 0;
    var max = count - 1;
    if (props.stuckAtBoundary) {
      min += (1 - slideRatio - offsetRatio) / slideRatio;
      max -= (1 - slideRatio - offsetRatio) / slideRatio;
    }
    return bound(cur, min, max);
  };
  var _useSpring = useSpring(function () {
      return {
        position: boundIndex(current) * 100,
        config: {
          tension: 200,
          friction: 30,
          autoplayInterval: autoplayInterval
        },
        onRest: function onRest() {
          if (draggingRef.current) return;
          var rawX = position.get();
          var totalWidth = 100 * count;
          var standardPosition = modulus(rawX, totalWidth);
          if (standardPosition === rawX) return;
          api.start({
            position: standardPosition,
            immediate: true
          });
        }
      };
    }, [count]),
    _useSpring2 = _slicedToArray(_useSpring, 2),
    position = _useSpring2[0].position,
    api = _useSpring2[1];
  var bind = useDrag(function (state) {
    if (_lock.current) return;
    var slidePixels = getSlidePixels();
    if (!slidePixels) return;
    var paramIndex = isVertical ? 1 : 0;
    var offset = state.offset[paramIndex];
    var itemDirection = state.direction[paramIndex];
    var velocity = state.velocity[paramIndex];
    setDragging(true);
    if (!state.last) {
      api.start({
        position: offset * 100 / slidePixels,
        immediate: true
      });
    } else {
      var index = Math.round((offset + Math.min(velocity * 2000, slidePixels) * itemDirection) / slidePixels);
      swipeTo(index);
      window.setTimeout(function () {
        if (mountedRef.current) {
          setDragging(false);
        }
      });
    }
  }, {
    transform: function transform(_ref) {
      var _ref2 = _slicedToArray(_ref, 2),
        x = _ref2[0],
        y = _ref2[1];
      return [-x, -y];
    },
    from: function from() {
      var slidePixels = getSlidePixels();
      return [position.get() / 100 * slidePixels, position.get() / 100 * slidePixels];
    },
    bounds: function bounds() {
      if (loop) return {};
      var slidePixels = getSlidePixels();
      var lowerBound = boundIndex(0) * slidePixels;
      var upperBound = boundIndex(count - 1) * slidePixels;
      return isVertical ? {
        top: lowerBound,
        bottom: upperBound
      } : {
        left: lowerBound,
        right: upperBound
      };
    },
    rubberband: true,
    axis: axis,
    preventScroll: !isVertical,
    pointer: {
      touch: true
    }
  });
  var renderIndicator = function renderIndicator() {
    if (props.indicator === undefined || props.indicator === true) {
      return /*#__PURE__*/React.createElement("div", {
        className: classnames(bem('indicator', {
          vertical: isVertical
        }))
      }, /*#__PURE__*/React.createElement(PageIndicator, _extends({}, props.indicatorProps, {
        direction: props.direction,
        total: count,
        current: current
      })));
    }
    if (typeof props.indicator === 'function') {
      return props.indicator(count, current);
    }
    return null;
  };
  var onClickCapture = function onClickCapture(e) {
    if (draggingRef.current) {
      e.stopPropagation();
    }
  };
  function swipeTo(index) {
    var immediate = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
    if (loop) {
      var _props$onIndexChange;
      var i = modulus(index, count);
      setCurrent(i);
      (_props$onIndexChange = props.onIndexChange) === null || _props$onIndexChange === void 0 ? void 0 : _props$onIndexChange.call(props, i);
      api.start({
        position: index * 100,
        immediate: immediate
      });
    } else {
      var _props$onIndexChange2;
      var _i = bound(index, 0, count - 1);
      setCurrent(_i);
      (_props$onIndexChange2 = props.onIndexChange) === null || _props$onIndexChange2 === void 0 ? void 0 : _props$onIndexChange2.call(props, _i);
      api.start({
        position: boundIndex(_i) * 100,
        immediate: immediate
      });
    }
  }
  var swipeNext = function swipeNext() {
    swipeTo(Math.round(position.get() / 100) + 1);
  };
  var swipePrev = function swipePrev() {
    swipeTo(Math.round(position.get() / 100) - 1);
  };
  useImperativeHandle(ref, function () {
    return {
      activeIndex: current,
      swipeTo: swipeTo,
      swipeNext: swipeNext,
      swipePrev: swipePrev,
      lock: function lock() {
        _lock.current = true;
      },
      unlock: function unlock() {
        _lock.current = false;
      }
    };
  });
  useEffect(function () {
    if (!autoplay || dragging) return noop;
    var interval = window.setInterval(function () {
      swipeNext();
    }, autoplayInterval);
    return function () {
      window.clearInterval(interval);
    };
  }, [autoplay, dragging, axisDistance]);
  if (count === 0) {
    devWarning('Swiper', '`Swiper` needs at least one child.');
  }
  return /*#__PURE__*/React.createElement("div", {
    ref: setRoot,
    className: classnames(props.className, bem({
      vertical: isVertical
    })),
    style: computedStyle
  }, /*#__PURE__*/React.createElement("div", _extends({
    ref: trackRef,
    className: classnames(bem('track', {
      'allow-touch-move': props.allowTouchMove
    })),
    onClickCapture: onClickCapture
  }, props.allowTouchMove ? bind() : {}), /*#__PURE__*/React.createElement("div", {
    className: classnames(bem('track-inner', {
      vertical: isVertical
    }))
  }, React.Children.map(validChildren, function (child, index) {
    var _ref3;
    return /*#__PURE__*/React.createElement(animated.div, {
      className: classnames(bem('slide')),
      style: (_ref3 = {}, _defineProperty(_ref3, axis, position.to(function (pos) {
        var finalPosition = -pos + index * 100;
        var totalWidth = count * 100;
        var flagWidth = totalWidth / 2;
        finalPosition = modulus(finalPosition + flagWidth, totalWidth) - flagWidth;
        return "".concat(finalPosition, "%");
      })), _defineProperty(_ref3, "left", "-".concat(index * 100, "%")), _ref3)
    }, child);
  }))), renderIndicator());
});
Swiper.defaultProps = {
  autoplay: false,
  autoplayInterval: 3000,
  allowTouchMove: true,
  defaultIndex: 0,
  loop: true,
  slideSize: 100,
  stuckAtBoundary: false,
  trackOffset: 0
};
Swiper.displayName = 'Swiper';
export default Swiper;