"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useLockScroll = useLockScroll;
function _react() {
  var data = require("react");
  _react = function _react() {
    return data;
  };
  return data;
}
var _useTouch = require("./use-touch");
var _useScrollParent = require("./use-scroll-parent");
var _useSupportPassive = require("./use-support-passive");
var totalLockCount = 0;
var BODY_LOCK_CLASS = 'rc-overflow-hidden';

// eslint-disable-next-line @typescript-eslint/explicit-module-boundary-types
function useLockScroll(rootRef, shouldLock) {
  var touch = (0, _useTouch.useTouch)();
  var onTouchMove = function onTouchMove(event) {
    touch.move(event);
    var direction = touch.deltaY.current > 0 ? '10' : '01';
    var el = (0, _useScrollParent.getScrollParent)(event.target, rootRef.current);
    if (!el) return;
    var scrollHeight = el.scrollHeight,
      offsetHeight = el.offsetHeight,
      scrollTop = el.scrollTop;
    var status = '11';
    if (scrollTop === 0) {
      status = offsetHeight >= scrollHeight ? '00' : '01';
    } else if (scrollTop + offsetHeight >= scrollHeight) {
      status = '10';
    }
    if (status !== '11' && touch.isVertical() &&
    // eslint-disable-next-line no-bitwise
    !(parseInt(status, 2) & parseInt(direction, 2))) {
      if (event.cancelable) {
        event.preventDefault();
      }
    }
  };
  var lock = function lock() {
    document.addEventListener('touchstart', touch.start);
    document.addEventListener('touchmove', onTouchMove, _useSupportPassive.supportsPassive ? {
      passive: false
    } : false);
    if (!totalLockCount) {
      document.body.classList.add(BODY_LOCK_CLASS);
    }

    // eslint-disable-next-line no-plusplus
    totalLockCount++;
  };
  var unlock = function unlock() {
    if (totalLockCount) {
      document.removeEventListener('touchstart', touch.start);
      document.removeEventListener('touchmove', onTouchMove);

      // eslint-disable-next-line no-plusplus
      totalLockCount--;
      if (!totalLockCount) {
        document.body.classList.remove(BODY_LOCK_CLASS);
      }
    }
  };

  // eslint-disable-next-line consistent-return
  (0, _react().useEffect)(function () {
    if (shouldLock) {
      lock();
      return function () {
        unlock();
      };
    }
  }, [shouldLock]);
}